@@ -10,14 +10,8 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Terrence R. Lambert.
- * 4. The name Terrence R. Lambert may not be used to endorse or promote
- *    products derived from this software without specific prior written
- *    permission.
  *
- * THIS SOFTWARE IS PROVIDED BY Julian R. Elischer ``AS IS'' AND ANY
+ * THIS SOFTWARE IS PROVIDED BY TAKASHI MANABE ``AS IS'' AND ANY
  * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL THE TERRENCE R. LAMBERT BE LIABLE
@@ -47,8 +41,15 @@
 #include	<string.h>
 #include	<unistd.h>
 #include	<sys/mman.h>
+#if defined(linux)
 /* #include	<linux/mm.h> */
 #include	<sys/kd.h>
+#elif defined(__FreeBSD__)
+#include      <vm/vm_param.h>
+#include      <sys/ioctl.h>
+#include      <machine/console.h>
+vm_size_t page_size;
+#endif
 #undef free
 #include	<stdlib.h>
 
@@ -223,7 +224,6 @@ void VgaInit(void)
     /* save font data in plane 2 */
     PortOutw(0x0204, VGAGRP_ADDR);
     memcpy(fontBuff1, gramMem, FONT_SIZE);
-    VgaLoadRomFont(fontBuff1);
     if (savePlane3) {
 	/* save font data in plane 3 */
 	PortOutw(0x0304, VGAGRP_ADDR);
@@ -338,8 +338,6 @@ void VgaSput(u_char *code, u_char fc, u_char bc)
     VgaOutByte(0xFF);
 }
 
-#ifndef	MINI_KON
-
 void VgaWputFm(u_char *code, u_char fc, u_char bc)
 {
     volatile char	*gram, *vcls;
@@ -395,8 +393,6 @@ void VgaSputFm(u_char *code, u_char fc, u_char bc)
     VgaOutByte(0xFF);
 }
 
-#endif /* MINI_KON */
-
 void VgaHardScrollUp(int line)
 {
     int	oldhead;
@@ -587,6 +583,7 @@ int VgaAttach(void)
 {
     int	devMem;
 
+#if defined(linux)
     ioperm(VGAMISC_IN, 1, 1);
     if (!(PortInb(VGAMISC_IN)&0x01)) { /* monochrome VGA */
 	vgaCrtAddr = 0x3B4;
@@ -614,19 +611,41 @@ int VgaAttach(void)
 	Perror("/dev/mem");
 	return FAILURE;
     }
+#elif defined(__FreeBSD__)
+    if (ioctl(0, KDENABIO,0) < 0) {
+	Perror("ioctl CONSOLE_IO_ENABLE");
+	return FAILURE;
+    }
+    if (ioctl(0, KDSETMODE,KD_GRAPHICS) < 0) {
+	Perror("ioctl CONSOLE_IO_ENABLE");
+	return FAILURE;
+    }
+    if ((devMem = open("/dev/vga", O_RDWR|O_NDELAY) ) < 0) {
+	Perror("/dev/mem");
+	return FAILURE;
+    }
+#endif
     if ((fontBuff1 = malloc(FONT_SIZE)) == NULL ||
 	(savePlane3 && (fontBuff2 = malloc(FONT_SIZE)) == NULL)) {
 	Perror("malloc ");
 	return FAILURE;
     }
     gramMem = (unsigned char *)mmap(
+#if defined(linux)
 				    (__ptr_t)0,
+#else
+				    0,
+#endif
 				    dInfo.gsize,
 				    PROT_READ|PROT_WRITE,
 #if 0
 				    MAP_SHARED|MAP_FIXED,
 #else
+#if defined(linux)
 				    MAP_SHARED,
+#elif defined(__FreeBSD__)
+				    MAP_FILE|MAP_SHARED,
+#endif
 #endif
 				    devMem,
 				    GRAPH_BASE
@@ -643,6 +662,7 @@ int VgaAttach(void)
 void VgaDetach(void)
 {
     gramHead = 0;
+#if defined(linux)
     ioperm(vgaCrtAddr, 1, 0);
     ioperm(VGAATTR_A_O, 1, 0);
     ioperm(VGAGRP_ADDR, 1, 0);
@@ -657,7 +677,8 @@ void VgaDetach(void)
     ioperm(VGAMISC_OUT, 1, 0);
     ioperm(vgaSt1Addr, 1, 0);
     ioperm(VGAPAL_DATA, 1, 0);
-    
+#endif
+
     munmap(gramMem, dInfo.gsize);
     
     SafeFree((void **)&gramMem);
@@ -705,8 +726,6 @@ void VgaDefaultCaps()
     DefineCap("CursorBottom", ConfigCursorBottom, "15");
 }
 
-#ifndef	MINI_KON
-
 int VgaFmSetVideoType(struct videoInfo *info, const char *regs)
 {
     union videoTimings video;
@@ -721,6 +740,4 @@ int VgaFmSetVideoType(struct videoInfo *info, const char *regs)
     return SUCCESS;
 }
 
-#endif /* MINI_KON */
-
 #endif
