@@ -11,15 +11,9 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Terrence R. Lambert.
- * 4. The name Terrence R. Lambert may not be used to endorse or promote
- *    products derived from this software without specific prior written
- *    permission.
  *
- * THIS SOFTWARE IS PROVIDED BY Julian R. Elischer ``AS IS'' AND ANY
- * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * THIS SOFTWARE IS PROVIDED BY MAEDA ATUSI AND TAKASHI MANABE ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL THE TERRENCE R. LAMBERT BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
@@ -37,12 +31,21 @@
 #include	<stdlib.h>
 #include	<string.h>
 #include	<unistd.h>
+#if defined(linux)
 #include	<sys/vt.h>
+#endif
 #include	<fcntl.h>
 #include	<signal.h>
+#if defined(linux)
 #include	<termio.h>
+#elif defined(__FreeBSD__)
+#include	<termios.h>
+#include	<machine/console.h>
+#endif
 #include	<sys/ioctl.h>
+#ifdef linux
 #include	<sys/kd.h>
+#endif
 
 #include	<mem.h>
 #include	<getcap.h>
@@ -270,7 +273,9 @@ void TextMode(void)
     vtm.relsig = 0;
     vtm.acqsig = 0;
     ioctl(0, VT_SETMODE, &vtm);
-    
+#if defined(__FreeBSD__)
+    ioctl(0, VT_RELDISP, 1);
+#endif
     con.text_mode = TRUE;
     SetTextMode();
     ioctl(masterPty, TIOCSWINSZ, &text_win);
@@ -283,6 +288,9 @@ void GraphMode(void)
 
     con.text_mode = FALSE;
     ioctl(0, KDSETMODE, KD_GRAPHICS);
+#if defined(__FreeBSD__)
+    ioctl(0, VT_RELDISP, VT_ACKACQ);
+#endif
     signal(SIGUSR1, LeaveVC);
     signal(SIGUSR2, EnterVC);
     vtm.mode = VT_PROCESS;
@@ -308,9 +316,10 @@ void GraphMode(void)
 static
     void	LeaveVC(int signum)
 {
+
     signal(SIGUSR1, LeaveVC);  /* should use sigaction()? */
     if (busy) {
-	release = TRUE;
+        release = TRUE;
 	return;
     }
     release = FALSE;
@@ -332,6 +341,7 @@ static
     if (!con.active) {
 	con.active = TRUE;
 	GraphMode();
+    signal(SIGUSR2, EnterVC);
 #ifdef	HAS_MOUSE
 	if (mInfo.has_mouse) {
 	    MouseStart();
@@ -746,12 +756,10 @@ static struct videoconf {
 } videos[] = {
 #ifdef	HAS_VGA
     {"VGA", SvgaSetVideoType},
-#ifndef	MINI_KON
     {"VGAFM", VgaFmSetVideoType},
     {"EGA", SvgaSetVideoType},
     {"SVGA", SvgaSetVideoType},
 #endif
-#endif
 #ifdef	HAS_S3
     {"S3", S3SetVideoType},
 #endif
@@ -808,17 +816,22 @@ static int	beepCount;
 
 static int	ConfigBeep(const char *confstr)
 {
-    if ((beepCount = atoi(confstr) * 10000) > 0)
+    beepCount = atoi(confstr) * 10000;
+#if defined(linux)
+    if (beepCount > 0)
 	ioperm(COUNTER_ADDR, 1, TRUE);
+#endif
     return SUCCESS;
 }
 
 void	Beep(void)
 {
     if (!con.active || beepCount <= 0) return;
+#if defined(linux)
     PortOutb(PortInb(COUNTER_ADDR)|3, COUNTER_ADDR);
     usleep(beepCount);
     PortOutb(PortInb(COUNTER_ADDR)&0xFC, COUNTER_ADDR);
+#endif
 }
 
 static int	ConfigInterval(const char *confstr)
@@ -867,7 +880,9 @@ void	ConsoleCleanup(void)
     SafeFree((void **)&textBuff);
     SafeFree((void **)&attrBuff);
     SafeFree((void **)&flagBuff);
+#ifdef linux
     ioperm(COUNTER_ADDR, 1, FALSE);
+#endif
     
     SafeFree((void **)&videoName);
 }
